

UserRam equ $00FF0000


; hello.asm

;		LIST	OFF;
;		INCLUDE	gen.h
;		LIST	ON
VDP_data	EQU	$C00000	; VDP data, R/W word or longword access only
VDP_ctrl	EQU	$C00004	; VDP control, word or longword writes only
TMSS_reg	EQU	$A14000	; (long) must store 'SEGA' if not version 0 hardware
HW_version	EQU	$A10001	; hardware version in low nibble
;-----------------------------------------------------------------------
;	exception vectors
;-----------------------------------------------------------------------


	DC.L	$FFFFFE00	; startup SP
	DC.L	START		; startup PC

	DS.L	7,RTEnn		; bus,addr,illegal,divzero,CHK,TRAPV,priv
	DC.L	RTEnn		; trace
	DC.L	RTEnn		; line 1010 emulator
	DC.L	RTEnn		; line 1111 emulator
	DS.L	4,RTEnn		; unassigned/uninitialized
	DS.L	8,RTEnn		; unassigned
	DC.L	RTEnn		; spurious interrupt
	DC.L	RTEnn		; interrupt level 1 (lowest priority)
	DC.L	ExtInt		; interrupt level 2 = external interrupt
	DC.L	RTEnn		; interrupt level 3
	DC.L	HSync		; interrupt level 4 = H-sync interrupt
	DC.L	RTEnn		; interrupt level 5
	DC.L	VSync		; interrupt level 6 = V-sync interrupt
	DC.L	RTEnn		; interrupt level 7 (highest priority)
	DS.L	16,RTEnn		; TRAP instruction vectors
	DS.L	16,RTEnn		; unassigned

;-----------------------------------------------------------------------
;	cartridge info header
;-----------------------------------------------------------------------

	DC.B	"SEGA GENESIS    "	; must start with "SEGA"
	DC.B	"(C)---- "		; copyright
 	DC.B	"2006.DEC"		; date
	DC.B	"HELLO WORLD                                     " ; cart name
	DC.B	"HELLO WORLD                                     " ; cart name (alt. language)
	DC.B	"GM MK-0000 -00"	; program type / catalog number
	DC.W	$0000			; ROM checksum
	DC.B	"J               "	; hardware used
	DC.L	$00000000		; start of ROM
	DC.L	$003FFFFF		; end of ROM
	DC.L	$00FF0000,$00FFFFFF	; RAM start/end
	DC.B	"            "		; backup RAM info
	DC.B	"            "		; modem info
	DC.B	"                                        " ; comment
	DC.B	"JUE             "	; regions allowed

;-----------------------------------------------------------------------
;	generic exception handler
;-----------------------------------------------------------------------

ExtInt
HSync
VSync
RTEnn
	RTE
;-----------------------------------------------------------------------
;	main entry point
;-----------------------------------------------------------------------

START
		LEA	Regs1(PC),A5	; initialize registers
		MOVEM.L	(A5)+,D5-D7/A0-A4

		; initialize TMSS
		MOVE.B	(A1),D0		; A10001 test the hardware version
		ANDI.B	#$0F,D0
		BEQ.B	.1		; branch if no TMSS

		MOVE.L	#'SEGA',(A2)	; A14000 disable TMSS

.1		MOVE.W	(A4),D0		; C00004 read VDP status (interrupt acknowledge?)

		; initialize USP and stuff
		MOVEQ	#0,D0		; D0 = 0
		MOVEA.L	D0,A6		; A6 = 0
		MOVE	A6,USP		; USP = 0

		MOVEQ	#24-1,D1	; length of video initialization block
.2		MOVE.B	(A5)+,D5	; get next video control byte
		MOVE.W	D5,(A4)		; C00004 send write register command to VDP
		ADD.W	D7,D5		; point to next VDP register
		DBRA	D1,.2		; loop for rest of block

		; DMA is now set up for 65535-byte fill of VRAM, let's do it
		MOVE.L	#$40000080,(A4)	; C00004 = VRAM write to $0000 (what is the 80 bit for?)
		MOVE.W	D0,(A3)		; C00000 = write zero to VRAM (starts DMA fill)

.3		MOVE.W	(A4),D4		; C00004 read VDP status
		BTST	#1,D4		; test DMA busy flag
		BNE.B	.3		; loop while DMA busy

		; initialize CRAM
		MOVE.L	#$81048F02,(A4)	; C00004 reg 1 = 0x04, reg 15 = 0x02: blank, auto-increment=2
		;MOVE.L	#$C0000000,(A4)	; C00004 write CRAM address $0000
		;MOVEQ	#32-1,D3	; loop for 32 CRAM registers
.4		;MOVE.L	D0,(A3)		; C00000 clear CRAM register
		;DBRA	D3,.4

		; initialize VSRAM
		;MOVE.L	#$40000010,(A4)	; C00004 VSRAM write address $0000
		;MOVEQ	#20-1,D4	; loop for 20 VSRAM registers
.5		;MOVE.L	D0,(A3)		; C00000 clear VSRAM register
		;DBRA	D4,.5

		; initialize PSG
		;MOVEQ	#4-1,D5		; loop for 4 PSG registers
.6		;MOVE.B	(A5)+,$0011(A3)	; C00011 copy PSG initialization commands
		;DBRA	D5,.6

		LEA	Regs2(PC),A1	; initialize registers
		MOVEM.L	(A1)+,D4-D7/A2-A6

		MOVE.W	(A2)+,D0		; get length word
		;MOVE.L	#$C0020000,(A4)		; C00004 CRAM write address = $0002
		MOVE.L	#$C0000000,(A4)		; C00004 CRAM write address = $0002
CramAgain:
		MOVE.W	(A2)+,(A5)		; C00000 write next word to video
		DBRA	D0,CramAgain			; loop until done

		
		move.L 	#96*8,d6
		lea Font,A1
		;MOVE.L	#$4C200000,(A4)	; C00004 VRAM write to $0C20
		MOVE.L	#$40000000,(A4)	; C00004 VRAM write to $0C20
		
NextFont:
		MOVE.B	(A1)+,d0
		
		move.L 	#0,d1
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		rol.L   #3,d1
		
		roxl.B	#1,d0
		roxl.L	#1,d1
		
		move.l d1,d0
		
		rol.l #1,d1
		or.l d0,d1
		rol.l #1,d1
		or.l d0,d1
		rol.l #1,d1
		or.l d0,d1
		
		;rol.L   #3,d1
		
		
		MOVE.L	d1,(A5)	; C00000 write next longword of charset to VDP
		;dbra D3,NextFont2
		
		
		DBRA	D6,NextFont		; loop until done

		MOVE.W	#$8144,(A4)	; C00004 reg 1 = 0x44 unblank display
